@@ -26,6 +26,7 @@ from tornado.log import app_log
 from traitlets.config import Application
 
 import jupyter_server
+from jupyter_server import CallContext
 from jupyter_server._sysinfo import get_sys_info
 from jupyter_server._tz import utcnow
 from jupyter_server.auth import authorized
@@ -224,6 +225,7 @@ class AuthenticatedHandler(web.RequestHandler):
                 "https://github.com/jupyter-server/jupyter_server/blob/"
                 "653740cbad7ce0c8a8752ce83e4d3c2c754b13cb/jupyter_server/serverapp.py"
                 "#L234-L256",
+                stacklevel=2,
             )
             from jupyter_server.auth import AllowAllAuthorizer
 
@@ -245,6 +247,7 @@ class AuthenticatedHandler(web.RequestHandler):
                 "add an identity provider to the tornado settings: "
                 "https://github.com/jupyter-server/jupyter_server/blob/v2.0.0/"
                 "jupyter_server/serverapp.py#L242",
+                stacklevel=2,
             )
             from jupyter_server.auth import IdentityProvider
 
@@ -580,6 +583,9 @@ class JupyterHandler(AuthenticatedHandler):
 
     async def prepare(self):
         """Pepare a response."""
+        # Set the current Jupyter Handler context variable.
+        CallContext.set(CallContext.JUPYTER_HANDLER, self)
+
         if not self.check_host():
             self.current_user = self._jupyter_current_user = None
             raise web.HTTPError(403)
@@ -594,10 +600,10 @@ class JupyterHandler(AuthenticatedHandler):
             # check for overridden get_current_user + default IdentityProvider
             # deprecated way to override auth (e.g. JupyterHub < 3.0)
             # allow deprecated, overridden get_current_user
-            warnings.warn(
+            warnings.warn(  # noqa
                 "Overriding JupyterHandler.get_current_user is deprecated in jupyter-server 2.0."
                 " Use an IdentityProvider class.",
-                DeprecationWarning,
+                DeprecationWarning
                 # stacklevel not useful here
             )
             user = self.get_current_user()
@@ -936,7 +942,13 @@ HTTPError = web.HTTPError
 
 
 class FileFindHandler(JupyterHandler, web.StaticFileHandler):
-    """subclass of StaticFileHandler for serving files from a search path"""
+    """subclass of StaticFileHandler for serving files from a search path
+
+    The setting "static_immutable_cache" can be set up to serve some static
+    file as immutable (e.g. file name containing a hash). The setting is a
+    list of base URL, every static file URL starting with one of those will
+    be immutable.
+    """
 
     # cache search results, don't search for files more than once
     _static_paths: dict = {}
@@ -945,8 +957,15 @@ class FileFindHandler(JupyterHandler, web.StaticFileHandler):
     def set_headers(self):
         """Set the headers."""
         super().set_headers()
+
+        immutable_paths = self.settings.get("static_immutable_cache", [])
+
+        # allow immutable cache for files
+        if any(self.request.path.startswith(path) for path in immutable_paths):
+            self.set_header("Cache-Control", "public, max-age=31536000, immutable")
+
         # disable browser caching, rely on 304 replies for savings
-        if "v" not in self.request.arguments or any(
+        elif "v" not in self.request.arguments or any(
             self.request.path.startswith(path) for path in self.no_cache_paths
         ):
             self.set_header("Cache-Control", "no-cache")
@@ -984,7 +1003,7 @@ class FileFindHandler(JupyterHandler, web.StaticFileHandler):
 
     def validate_absolute_path(self, root, absolute_path):
         """check if the file should be served (raises 404, 403, etc.)"""
-        if absolute_path == "":
+        if not absolute_path:
             raise web.HTTPError(404)
 
         for root in self.root:
