@@ -8,8 +8,6 @@ from tornado.websocket import WebSocketHandler
 from jupyter_server.base.handlers import JupyterHandler
 from jupyter_server.base.websocket import WebSocketMixin
 
-from .handlers import _kernel_id_regex
-
 AUTH_RESOURCE = "kernels"
 
 
@@ -20,6 +18,7 @@ class KernelWebsocketHandler(WebSocketMixin, WebSocketHandler, JupyterHandler):
 
     @property
     def kernel_websocket_connection_class(self):
+        """The kernel websocket connection class."""
         return self.settings.get("kernel_websocket_connection_class")
 
     def set_default_headers(self):
@@ -30,9 +29,11 @@ class KernelWebsocketHandler(WebSocketMixin, WebSocketHandler, JupyterHandler):
         pass
 
     def get_compression_options(self):
+        """Get the socket connection options."""
         return self.settings.get("websocket_compression_options", None)
 
     async def pre_get(self):
+        """Handle a pre_get."""
         # authenticate first
         user = self.current_user
         if user is None:
@@ -58,11 +59,16 @@ class KernelWebsocketHandler(WebSocketMixin, WebSocketHandler, JupyterHandler):
             await self.connection.prepare()
 
     async def get(self, kernel_id):
+        """Handle a get request for a kernel."""
         self.kernel_id = kernel_id
         await self.pre_get()
         await super().get(kernel_id=kernel_id)
 
     async def open(self, kernel_id):
+        """Open a kernel websocket."""
+        # Need to call super here to make sure we
+        # begin a ping-pong loop with the client.
+        super().open()
         # Wait for the kernel to emit an idle status.
         self.log.info(f"Connecting to kernel {self.kernel_id}.")
         await self.connection.connect()
@@ -72,20 +78,17 @@ class KernelWebsocketHandler(WebSocketMixin, WebSocketHandler, JupyterHandler):
         self.connection.handle_incoming_message(ws_message)
 
     def on_close(self):
+        """Handle a socket closure."""
         self.connection.disconnect()
         self.connection = None
 
     def select_subprotocol(self, subprotocols):
+        """Select the sub protocol for the socket."""
         preferred_protocol = self.connection.kernel_ws_protocol
         if preferred_protocol is None:
             preferred_protocol = "v1.kernel.websocket.jupyter.org"
-        elif preferred_protocol == "":
+        elif preferred_protocol == "":  # noqa
             preferred_protocol = None
         selected_subprotocol = preferred_protocol if preferred_protocol in subprotocols else None
         # None is the default, "legacy" protocol
         return selected_subprotocol
-
-
-default_handlers = [
-    (r"/api/kernels/%s/channels" % _kernel_id_regex, KernelWebsocketHandler),
-]
